/*
 * Galatea Dialog Manager:
 * (c)2003 Takuya NISHIMOTO (nishi@hil.t.u-tokyo.ac.jp)
 * $Id: RecogInterpreterListener.java,v 1.1 2006/11/16 02:39:40 nishi Exp $
 */

package galatea.recogman;

public interface RecogInterpreterListener
{
    public void recogListenStarted();
    public void recogRecordStarted();
    public void recogRecordFinished();
    public void recogPass1Finished();
    public void recogPass2Finished();
    public void recogFailed();
    public void recogException(Exception e, String str);
}
